@@ -33,6 +33,7 @@
 #include <unistd.h>
 #endif
 
+#include "gdk-pixbuf-alias.h"
 #include "gdk-pixbuf-private.h"
 #include "gdk-pixbuf-io.h"
 
@@ -75,15 +76,36 @@ format_check (GdkPixbufModule *module, guchar *buffer, int size)
 	return 0;
 }
 
+G_LOCK_DEFINE_STATIC (init_lock);
+G_LOCK_DEFINE_STATIC (threadunsafe_loader_lock);
+
+void
+_gdk_pixbuf_lock (GdkPixbufModule *image_module)
+{
+ 	if (!(image_module->info->flags & GDK_PIXBUF_FORMAT_THREADSAFE)) {
+ 		G_LOCK (threadunsafe_loader_lock);
+ 	}
+}
+ 
+void
+_gdk_pixbuf_unlock (GdkPixbufModule *image_module)
+{
+	if (!(image_module->info->flags & GDK_PIXBUF_FORMAT_THREADSAFE)) {
+		G_UNLOCK (threadunsafe_loader_lock);
+	}
+}
+
 static GSList *file_formats = NULL;
 
 static void gdk_pixbuf_io_init ();
 
 static GSList *
-get_file_formats ()
+get_file_formats (void)
 {
+	G_LOCK (init_lock);
 	if (file_formats == NULL)
 		gdk_pixbuf_io_init ();
+	G_UNLOCK (init_lock);
 	
 	return file_formats;
 }
@@ -235,7 +257,7 @@ gdk_pixbuf_get_module_file (void)
 }
 
 static void 
-gdk_pixbuf_io_init ()
+gdk_pixbuf_io_init (void)
 {
 	GIOChannel *channel;
 	gchar *line_buf;
@@ -251,7 +273,7 @@ gdk_pixbuf_io_init ()
 
 	channel = g_io_channel_new_file (filename, "r",  &error);
 	if (!channel) {
-		g_warning ("Can not open pixbuf loader module file '%s': %s",
+		g_warning ("Cannot open pixbuf loader module file '%s': %s",
 			   filename, error->message);
 		return;
 	}
@@ -333,7 +355,7 @@ gdk_pixbuf_io_init ()
 					module->info->mime_types =
 						g_realloc (module->info->mime_types, (n + 1) * sizeof (gchar*));
 					module->info->mime_types[n - 1] = g_strdup (tmp_buf->str);
-					module->info->mime_types[n] = 0;
+					module->info->mime_types[n] = NULL;
 					n++;
 				}
 			}
@@ -346,7 +368,7 @@ gdk_pixbuf_io_init ()
 					module->info->extensions =
 						g_realloc (module->info->extensions, (n + 1) * sizeof (gchar*));
 					module->info->extensions[n - 1] = g_strdup (tmp_buf->str);
-					module->info->extensions[n] = 0;
+					module->info->extensions[n] = NULL;
 					n++;
 				}
 			}
@@ -395,9 +417,9 @@ gdk_pixbuf_io_init ()
 /* actually load the image handler - gdk_pixbuf_get_module only get a */
 /* reference to the module to load, it doesn't actually load it       */
 /* perhaps these actions should be combined in one function           */
-gboolean
-_gdk_pixbuf_load_module (GdkPixbufModule *image_module,
-                         GError         **error)
+static gboolean
+_gdk_pixbuf_load_module_unlocked (GdkPixbufModule *image_module,
+				  GError         **error)
 {
 	char *path;
 	GModule *module;
@@ -432,6 +454,19 @@ _gdk_pixbuf_load_module (GdkPixbufModule *image_module,
                 return FALSE;
         }
 }
+
+gboolean
+_gdk_pixbuf_load_module (GdkPixbufModule *image_module,
+			 GError         **error)
+{
+	gboolean ret;
+
+	G_LOCK (init_lock);
+	ret = _gdk_pixbuf_load_module_unlocked (image_module, error);
+	G_UNLOCK (init_lock);
+
+	return ret;
+}
 #else
 
 #define module(type) \
@@ -592,11 +627,11 @@ gdk_pixbuf_io_init ()
 	};
 	gchar **name;
 	GdkPixbufModule *module = NULL;
-	
+
 	for (name = included_formats; *name; name++) {
 		module = g_new0 (GdkPixbufModule, 1);
 		module->module_name = *name;
-		if (_gdk_pixbuf_load_module (module, NULL))
+		if (_gdk_pixbuf_load_module_unlocked (module, NULL))
 			file_formats = g_slist_prepend (file_formats, module);
 		else
 			g_free (module);
@@ -615,6 +650,10 @@ _gdk_pixbuf_get_named_module (const char *name,
 
 	for (modules = get_file_formats (); modules; modules = g_slist_next (modules)) {
 		GdkPixbufModule *module = (GdkPixbufModule *)modules->data;
+
+		if (module->info->disabled)
+			continue;
+
 		if (!strcmp (name, module->module_name))
 			return module;
 	}
@@ -637,10 +676,14 @@ _gdk_pixbuf_get_module (guchar *buffer, guint size,
 
 	gint score, best = 0;
 	GdkPixbufModule *selected = NULL;
-	gchar *utf8_filename = NULL;
+	gchar *display_name = NULL;
 
 	for (modules = get_file_formats (); modules; modules = g_slist_next (modules)) {
 		GdkPixbufModule *module = (GdkPixbufModule *)modules->data;
+
+		if (module->info->disabled)
+			continue;
+
 		score = format_check (module, buffer, size);
 		if (score > best) {
 			best = score; 
@@ -653,16 +696,14 @@ _gdk_pixbuf_get_module (guchar *buffer, guint size,
 		return selected;
 
         if (filename)
-		utf8_filename = g_filename_to_utf8 (filename, -1,
-						    NULL, NULL, NULL);
-	
-	if (utf8_filename) {
+	{
+		display_name = g_filename_display_name (filename);
 		g_set_error (error,
 			     GDK_PIXBUF_ERROR,
 			     GDK_PIXBUF_ERROR_UNKNOWN_TYPE,
 			     _("Couldn't recognize the image file format for file '%s'"),
-			     utf8_filename);
-		g_free (utf8_filename);
+			     display_name);
+		g_free (display_name);
 	}
         else
                 g_set_error (error,
@@ -696,37 +737,37 @@ _gdk_pixbuf_generic_image_load (GdkPixbufModule *module,
 	GdkPixbufAnimation *animation = NULL;
 	gpointer context;
 
-	if (module->load != NULL)
-		return (* module->load) (f, error);
-	
-	if (module->begin_load != NULL) {
+	_gdk_pixbuf_lock (module);
+
+	if (module->load != NULL) {
+		pixbuf = (* module->load) (f, error);
+	} else if (module->begin_load != NULL) {
 		
 		context = module->begin_load (NULL, prepared_notify, NULL, &pixbuf, error);
 	
 		if (!context)
-			return NULL;
+			goto out;
 		
 		while (!feof (f) && !ferror (f)) {
 			length = fread (buffer, 1, sizeof (buffer), f);
 			if (length > 0)
 				if (!module->load_increment (context, buffer, length, error)) {
 					module->stop_load (context, NULL);
-					if (pixbuf != NULL)
+					if (pixbuf != NULL) {
 						g_object_unref (pixbuf);
-					return NULL;
+						pixbuf = NULL;
+					}
+					goto out;
 				}
 		}
 		
 		if (!module->stop_load (context, error)) {
-			if (pixbuf != NULL)
+			if (pixbuf != NULL) {
 				g_object_unref (pixbuf);
-			return NULL;
+				pixbuf = NULL;
+			}
 		}
-		
-		return pixbuf;
-	}
-	
-	if (module->load_animation != NULL) {
+	} else if (module->load_animation != NULL) {
 		animation = (* module->load_animation) (f, error);
 		if (animation != NULL) {
 			pixbuf = gdk_pixbuf_animation_get_static_image (animation);
@@ -734,12 +775,12 @@ _gdk_pixbuf_generic_image_load (GdkPixbufModule *module,
 			g_object_ref (pixbuf);
 
 			g_object_unref (animation);
-			
-			return pixbuf;
 		}
 	}
 
-	return NULL;
+ out:
+	_gdk_pixbuf_unlock (module);
+	return pixbuf;
 }
 
 /**
@@ -765,46 +806,47 @@ gdk_pixbuf_new_from_file (const char *filename,
 	FILE *f;
 	guchar buffer [128];
 	GdkPixbufModule *image_module;
+	gchar *display_name;
 
 	g_return_val_if_fail (filename != NULL, NULL);
         g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+	
+	display_name = g_filename_display_name (filename);	
 
 	f = fopen (filename, "rb");
 	if (!f) {
-                gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-                                                           NULL, NULL, NULL);
                 g_set_error (error,
                              G_FILE_ERROR,
                              g_file_error_from_errno (errno),
                              _("Failed to open file '%s': %s"),
-                             utf8_filename ? utf8_filename : "???",
+                             display_name,
                              g_strerror (errno));
-                g_free (utf8_filename);
+                g_free (display_name);
 		return NULL;
         }
 
 	size = fread (&buffer, 1, sizeof (buffer), f);
 	if (size == 0) {
-                gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-                                                           NULL, NULL, NULL);
                 g_set_error (error,
                              GDK_PIXBUF_ERROR,
                              GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
                              _("Image file '%s' contains no data"),
-                             utf8_filename ? utf8_filename : "???");
-                g_free (utf8_filename);
+                             display_name);
+                g_free (display_name);
 		fclose (f);
 		return NULL;
 	}
 
 	image_module = _gdk_pixbuf_get_module (buffer, size, filename, error);
         if (image_module == NULL) {
+                g_free (display_name);
                 fclose (f);
                 return NULL;
         }
 
 	if (image_module->module == NULL)
                 if (!_gdk_pixbuf_load_module (image_module, error)) {
+			g_free (display_name);
                         fclose (f);
                         return NULL;
                 }
@@ -821,34 +863,26 @@ gdk_pixbuf_new_from_file (const char *filename,
                  * the invariant that error gets set if NULL is returned.
                  */
 
-                gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-                                                           NULL, NULL, NULL);
-
                 g_warning ("Bug! gdk-pixbuf loader '%s' didn't set an error on failure.", image_module->module_name);
                 g_set_error (error,
                              GDK_PIXBUF_ERROR,
                              GDK_PIXBUF_ERROR_FAILED,
                              _("Failed to load image '%s': reason not known, probably a corrupt image file"),
-                             utf8_filename ? utf8_filename : "???");
-                g_free (utf8_filename);
+                             display_name);
         } else if (error != NULL && *error != NULL) {
 
           /* Add the filename to the error message */
           GError *e = *error;
           gchar  *old;
-          gchar  *utf8_filename = g_filename_to_utf8 (filename, -1,
-                                                      NULL, NULL, NULL);
 
           old = e->message;
-
           e->message = g_strdup_printf (_("Failed to load image '%s': %s"),
-                                        utf8_filename ? utf8_filename : "???",
+                                        display_name,
                                         old);
-
-          g_free (utf8_filename);
           g_free (old);
         }
 
+	g_free (display_name);
 	return pixbuf;
 }
 
@@ -859,49 +893,59 @@ size_prepared_cb (GdkPixbufLoader *loader,
 		  gpointer         data)
 {
 	struct {
-		int width;
-		int height;
+		gint width;
+		gint height;
+		gboolean preserve_aspect_ratio;
 	} *info = data;
 
 	g_return_if_fail (width > 0 && height > 0);
 
-	if ((double)height * (double)info->width >
-	    (double)width * (double)info->height) {
-		width = 0.5 + (double)width * (double)info->height / (double)height;
-		height = info->height;
+	if(info->preserve_aspect_ratio) {
+		if ((double)height * (double)info->width >
+		    (double)width * (double)info->height) {
+			width = 0.5 + (double)width * (double)info->height / (double)height;
+			height = info->height;
+		} else {
+			height = 0.5 + (double)height * (double)info->width / (double)width;
+			width = info->width;
+		}
 	} else {
-		height = 0.5 + (double)height * (double)info->width / (double)width;
 		width = info->width;
+		height = info->height;
 	}
-
+	
 	gdk_pixbuf_loader_set_size (loader, width, height);
 }
 
 /**
- * gdk_pixbuf_new_from_file_at_size:
+ * gdk_pixbuf_new_from_file_at_scale:
  * @filename: Name of file to load.
  * @width: The width the image should have
  * @height: The height the image should have
+ * @preserve_aspect_ratio: %TRUE to preserve the image's aspect ratio
  * @error: Return location for an error
  *
  * Creates a new pixbuf by loading an image from a file.  The file format is
  * detected automatically. If %NULL is returned, then @error will be set.
  * Possible errors are in the #GDK_PIXBUF_ERROR and #G_FILE_ERROR domains.
- * The image will be scaled to fit in the requested size, preserving its aspect ratio.
+ * The image will be scaled to fit in the requested size, optionally preserving
+ * the image's aspect ratio.
  *
- * Return value: A newly-created pixbuf with a reference count of 1, or %NULL if
- * any of several error conditions occurred:  the file could not be opened,
+ * Return value: A newly-created pixbuf with a reference count of 1, or %NULL 
+ * if any of several error conditions occurred:  the file could not be opened,
  * there was no loader for the file's format, there was not enough memory to
  * allocate the image buffer, or the image file contained invalid data.
  *
- * Since: 2.4
+ * Since: 2.6
  **/
 GdkPixbuf *
-gdk_pixbuf_new_from_file_at_size (const char *filename,
-				  int         width, 
-				  int         height,
-				  GError    **error)
+gdk_pixbuf_new_from_file_at_scale (const char *filename,
+				   int         width, 
+				   int         height,
+				   gboolean    preserve_aspect_ratio,
+				   GError    **error)
 {
+
 	GdkPixbufLoader *loader;
 	GdkPixbuf       *pixbuf;
 
@@ -911,6 +955,7 @@ gdk_pixbuf_new_from_file_at_size (const char *filename,
 	struct {
 		gint width;
 		gint height;
+		gboolean preserve_aspect_ratio;
 	} info;
 
 	g_return_val_if_fail (filename != NULL, NULL);
@@ -918,15 +963,14 @@ gdk_pixbuf_new_from_file_at_size (const char *filename,
 
 	f = fopen (filename, "rb");
 	if (!f) {
-                gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-                                                           NULL, NULL, NULL);
+                gchar *display_name = g_filename_display_name (filename);
                 g_set_error (error,
                              G_FILE_ERROR,
                              g_file_error_from_errno (errno),
                              _("Failed to open file '%s': %s"),
-                             utf8_filename ? utf8_filename : "???",
+                             display_name,
                              g_strerror (errno));
-                g_free (utf8_filename);
+                g_free (display_name);
 		return NULL;
         }
 
@@ -934,6 +978,7 @@ gdk_pixbuf_new_from_file_at_size (const char *filename,
 
 	info.width = width;
 	info.height = height;
+        info.preserve_aspect_ratio = preserve_aspect_ratio;
 
 	g_signal_connect (loader, "size-prepared", G_CALLBACK (size_prepared_cb), &info);
 
@@ -958,17 +1003,14 @@ gdk_pixbuf_new_from_file_at_size (const char *filename,
 	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 
 	if (!pixbuf) {
-                gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-                                                           NULL, NULL, NULL);
-
+                gchar *display_name = g_filename_display_name (filename);
 		g_object_unref (loader);
-
                 g_set_error (error,
                              GDK_PIXBUF_ERROR,
                              GDK_PIXBUF_ERROR_FAILED,
                              _("Failed to load image '%s': reason not known, probably a corrupt image file"),
-                             utf8_filename ? utf8_filename : "???");
-                g_free (utf8_filename);
+                             display_name);
+                g_free (display_name);
 		return NULL;
 	}
 
@@ -979,6 +1021,38 @@ gdk_pixbuf_new_from_file_at_size (const char *filename,
 	return pixbuf;
 }
 
+/**
+ * gdk_pixbuf_new_from_file_at_size:
+ * @filename: Name of file to load.
+ * @width: The width the image should have
+ * @height: The height the image should have
+ * @error: Return location for an error
+ *
+ * Creates a new pixbuf by loading an image from a file.  The file format is
+ * detected automatically. If %NULL is returned, then @error will be set.
+ * Possible errors are in the #GDK_PIXBUF_ERROR and #G_FILE_ERROR domains.
+ * The image will be scaled to fit in the requested size, preserving
+ * the image's aspect ratio.
+ *
+ * Return value: A newly-created pixbuf with a reference count of 1, or 
+ * %NULL if any of several error conditions occurred:  the file could not 
+ * be opened, there was no loader for the file's format, there was not 
+ * enough memory to allocate the image buffer, or the image file contained 
+ * invalid data.
+ *
+ * Since: 2.4
+ **/
+GdkPixbuf *
+gdk_pixbuf_new_from_file_at_size (const char *filename,
+				  int         width, 
+				  int         height,
+				  GError    **error)
+{
+	return gdk_pixbuf_new_from_file_at_scale (filename, 
+						  width, height, 
+						  TRUE, error);
+}
+
 static void
 info_cb (GdkPixbufLoader *loader, 
 	 int              width,
@@ -1094,14 +1168,18 @@ gdk_pixbuf_new_from_xpm_data (const char **data)
                         return NULL;
                 }
         }
-          
+
+	_gdk_pixbuf_lock (xpm_module);
+
 	if (xpm_module->load_xpm_data == NULL) {
 		g_warning ("gdk-pixbuf XPM module lacks XPM data capability");
-		return NULL;
-	} else
+		pixbuf = NULL;
+	} else {
 		load_xpm_data = xpm_module->load_xpm_data;
+		pixbuf = (* load_xpm_data) (data);
+	}
 
-	pixbuf = (* load_xpm_data) (data);
+	_gdk_pixbuf_unlock (xpm_module);
 	return pixbuf;
 }
 
@@ -1170,39 +1248,43 @@ gdk_pixbuf_real_save (GdkPixbuf     *pixbuf,
                       gchar        **values,
                       GError       **error)
 {
-       GdkPixbufModule *image_module = NULL;       
+	gboolean ret;
+	GdkPixbufModule *image_module = NULL;       
 
-       image_module = _gdk_pixbuf_get_named_module (type, error);
+	image_module = _gdk_pixbuf_get_named_module (type, error);
 
-       if (image_module == NULL)
-               return FALSE;
+	if (image_module == NULL)
+		return FALSE;
        
-       if (image_module->module == NULL)
-               if (!_gdk_pixbuf_load_module (image_module, error))
-                       return FALSE;
+	if (image_module->module == NULL)
+		if (!_gdk_pixbuf_load_module (image_module, error))
+			return FALSE;
 
-       if (image_module->save) {
-	       /* save normally */
-	       return (* image_module->save) (filehandle, pixbuf,
+	_gdk_pixbuf_lock (image_module);
+
+	if (image_module->save) {
+		/* save normally */
+		ret = (* image_module->save) (filehandle, pixbuf,
 					      keys, values,
 					      error);
-       }
-       else if (image_module->save_to_callback) {
-	       /* save with simple callback */
-	       return (* image_module->save_to_callback) (save_to_file_callback,
+	} else if (image_module->save_to_callback) {
+		/* save with simple callback */
+		ret = (* image_module->save_to_callback) (save_to_file_callback,
 							  filehandle, pixbuf,
 							  keys, values,
 							  error);
-       }
-       else {
-	       /* can't save */
-               g_set_error (error,
-                            GDK_PIXBUF_ERROR,
-                            GDK_PIXBUF_ERROR_UNSUPPORTED_OPERATION,
-                            _("This build of gdk-pixbuf does not support saving the image format: %s"),
-                            type);
-               return FALSE;
-       }
+	} else {
+		/* can't save */
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+			     GDK_PIXBUF_ERROR_UNSUPPORTED_OPERATION,
+			     _("This build of gdk-pixbuf does not support saving the image format: %s"),
+			     type);
+		ret = FALSE;
+	}
+
+	_gdk_pixbuf_unlock (image_module);
+	return ret;
 }
 
 #define TMP_FILE_BUF_SIZE 4096
@@ -1243,8 +1325,13 @@ save_to_callback_with_tmp_file (GdkPixbufModule   *image_module,
 			     _("Failed to open temporary file"));
 		goto end;
 	}
-	if (!(* image_module->save) (f, pixbuf, keys, values, error))
+
+	_gdk_pixbuf_lock (image_module);
+	retval = (image_module->save) (f, pixbuf, keys, values, error);
+	_gdk_pixbuf_unlock (image_module);
+	if (!retval)
 		goto end;
+
 	rewind (f);
 	for (;;) {
 		n = fread (buf, 1, TMP_FILE_BUF_SIZE, f);
@@ -1286,39 +1373,43 @@ gdk_pixbuf_real_save_to_callback (GdkPixbuf         *pixbuf,
 				  gchar            **values,
 				  GError           **error)
 {
-       GdkPixbufModule *image_module = NULL;       
+	gboolean ret;
+	GdkPixbufModule *image_module = NULL;       
 
-       image_module = _gdk_pixbuf_get_named_module (type, error);
+	image_module = _gdk_pixbuf_get_named_module (type, error);
 
-       if (image_module == NULL)
-               return FALSE;
+	if (image_module == NULL)
+		return FALSE;
        
-       if (image_module->module == NULL)
-               if (!_gdk_pixbuf_load_module (image_module, error))
-                       return FALSE;
+	if (image_module->module == NULL)
+		if (!_gdk_pixbuf_load_module (image_module, error))
+			return FALSE;
 
-       if (image_module->save_to_callback) {
-	       /* save normally */
-	       return (* image_module->save_to_callback) (save_func, user_data, 
+	_gdk_pixbuf_lock (image_module);
+
+	if (image_module->save_to_callback) {
+		/* save normally */
+		ret = (* image_module->save_to_callback) (save_func, user_data, 
 							  pixbuf, keys, values,
 							  error);
-       }
-       else if (image_module->save) {
-	       /* use a temporary file */
-	       return save_to_callback_with_tmp_file (image_module, pixbuf,
+	} else if (image_module->save) {
+		/* use a temporary file */
+		ret = save_to_callback_with_tmp_file (image_module, pixbuf,
 						      save_func, user_data, 
 						      keys, values,
 						      error);
-       }
-       else {
-	       /* can't save */
-               g_set_error (error,
-                            GDK_PIXBUF_ERROR,
-                            GDK_PIXBUF_ERROR_UNSUPPORTED_OPERATION,
-                            _("This build of gdk-pixbuf does not support saving the image format: %s"),
-                            type);
-               return FALSE;
-       }
+	} else {
+		/* can't save */
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+			     GDK_PIXBUF_ERROR_UNSUPPORTED_OPERATION,
+			     _("This build of gdk-pixbuf does not support saving the image format: %s"),
+			     type);
+		ret = FALSE;
+	}
+
+	_gdk_pixbuf_unlock (image_module);
+	return ret;
 }
 
  
@@ -1436,15 +1527,14 @@ gdk_pixbuf_savev (GdkPixbuf  *pixbuf,
         f = fopen (filename, "wb");
         
         if (f == NULL) {
-                gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-                                                           NULL, NULL, NULL);
+                gchar *display_name = g_filename_display_name (filename);
                 g_set_error (error,
                              G_FILE_ERROR,
                              g_file_error_from_errno (errno),
                              _("Failed to open '%s' for writing: %s"),
-                             utf8_filename ? utf8_filename : "???",
+                             display_name,
                              g_strerror (errno));
-                g_free (utf8_filename);
+                g_free (display_name);
                 return FALSE;
         }
 
@@ -1461,15 +1551,14 @@ gdk_pixbuf_savev (GdkPixbuf  *pixbuf,
        }
 
        if (fclose (f) < 0) {
-               gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-                                                           NULL, NULL, NULL);
+               gchar *display_name = g_filename_display_name (filename);
                g_set_error (error,
                             G_FILE_ERROR,
                             g_file_error_from_errno (errno),
                             _("Failed to close '%s' while writing image, all data may not have been saved: %s"),
-                            utf8_filename ? utf8_filename : "???",
+                            display_name,
                             g_strerror (errno));
-               g_free (utf8_filename);
+               g_free (display_name);
                return FALSE;
        }
        
@@ -1822,6 +1911,88 @@ gdk_pixbuf_format_is_writable (GdkPixbufFormat *format)
 	return (format->flags & GDK_PIXBUF_FORMAT_WRITABLE) != 0;
 }
 
+/**
+ * gdk_pixbuf_format_is_scalable:
+ * @format: a #GdkPixbufFormat
+ *
+ * Returns whether this image format is scalable. If a file is in a 
+ * scalable format, it is preferable to load it at the desired size, 
+ * rather than loading it at the default size and scaling the 
+ * resulting pixbuf to the desired size.
+ * 
+ * Return value: whether this image format is scalable.
+ *
+ * Since: 2.6
+ */
+gboolean
+gdk_pixbuf_format_is_scalable (GdkPixbufFormat *format)
+{
+	g_return_val_if_fail (format != NULL, FALSE);
+
+	return (format->flags & GDK_PIXBUF_FORMAT_SCALABLE) != 0;
+}
+
+/**
+ * gdk_pixbuf_format_is_disabled:
+ * @format: a #GdkPixbufFormat
+ *
+ * Returns whether this image format is disabled. See
+ * gdk_pixbuf_format_set_disabled().
+ * 
+ * Return value: whether this image format is disabled.
+ *
+ * Since: 2.6
+ */
+gboolean   
+gdk_pixbuf_format_is_disabled (GdkPixbufFormat *format)
+{
+	g_return_val_if_fail (format != NULL, FALSE);
+
+	return format->disabled;	
+}
+
+/**
+ * gdk_pixbuf_format_set_disabled:
+ * @format: a #GdkPixbufFormat
+ * @disabled: %TRUE to disable the format @format
+ *
+ * Disables or enables an image format. If a format is disabled, 
+ * gdk-pixbuf won't use the image loader for this format to load 
+ * images. Applications can use this to avoid using image loaders 
+ * with an inappropriate license, see gdk_pixbuf_format_get_license().
+ *
+ * Since: 2.6
+ */
+void 
+gdk_pixbuf_format_set_disabled (GdkPixbufFormat *format,
+				gboolean         disabled)
+{
+	g_return_if_fail (format != NULL);
+	
+	format->disabled = disabled != FALSE;
+}
+
+/**
+ * gdk_pixbuf_format_get_license:
+ * @format: a #GdkPixbufFormat
+ *
+ * Returns information about the license of the image loader
+ * for the format. The returned string should be a shorthand for 
+ * a wellknown license, e.g. "LGPL", "GPL", "QPL", "GPL/QPL",
+ * or "other" to indicate some other license.  
+ *
+ * Returns: a string describing the license of @format. 
+ *
+ * Since: 2.6
+ */
+gchar*
+gdk_pixbuf_format_get_license (GdkPixbufFormat *format)
+{
+	g_return_val_if_fail (format != NULL, NULL);
+
+	return g_strdup (format->license);
+}
+
 GdkPixbufFormat *
 _gdk_pixbuf_get_format (GdkPixbufModule *module)
 {
