@@ -112,6 +112,9 @@ static gboolean gdk_event_dispatch (GSource     *source,
 				    GSourceFunc  callback,
 				    gpointer     user_data);
 
+static void append_event (GdkDisplay *display,
+			  GdkEvent   *event);
+
 /* Private variable declarations
  */
 
@@ -159,7 +162,6 @@ static HKL latin_locale = NULL;
 #endif
 
 static gboolean in_ime_composition = FALSE;
-static gboolean resizing = FALSE;
 static UINT     resize_timer;
 
 static int debug_indent = 0;
@@ -226,11 +228,24 @@ _gdk_win32_get_next_tick (gulong suggested_tick)
   if (suggested_tick == 0)
     suggested_tick = GetTickCount ();
   if (suggested_tick <= cur_tick)
-    return ++cur_tick;
+    return cur_tick;
   else
     return cur_tick = suggested_tick;
 }
 
+static void
+generate_focus_event (GdkWindow *window,
+		      gboolean   in)
+{
+  GdkEvent *event;
+
+  event = gdk_event_new (GDK_FOCUS_CHANGE);
+  event->focus_change.window = window;
+  event->focus_change.in = in;
+
+  append_event (gdk_drawable_get_display (window), event);
+}
+
 static LRESULT 
 inner_window_procedure (HWND   hwnd,
 			UINT   message,
@@ -717,6 +732,8 @@ gdk_keyboard_grab (GdkWindow *window,
 		   gboolean   owner_events,
 		   guint32    time)
 {
+  GdkWindow *real_focus_window, *grab_focus_window;
+
   gint return_val;
   
   g_return_val_if_fail (window != NULL, 0);
@@ -734,7 +751,31 @@ gdk_keyboard_grab (GdkWindow *window,
     return_val = GDK_GRAB_ALREADY_GRABBED;
 
   if (return_val == GDK_GRAB_SUCCESS)
-    k_grab_window = window;
+    {
+      k_grab_window = window;
+
+      if (!k_grab_owner_events)
+	{
+	  real_focus_window = gdk_win32_handle_table_lookup ((GdkNativeWindow) GetFocus ());
+	  if (real_focus_window)
+	    real_focus_window = gdk_window_get_toplevel (real_focus_window);
+	  grab_focus_window = gdk_window_get_toplevel (k_grab_window);
+	  if (real_focus_window != grab_focus_window)
+	    {
+	      /* Generate events for focus change from the window that really
+	       * has focus to the grabber.
+	       */
+	      if (real_focus_window && !GDK_WINDOW_DESTROYED (real_focus_window)
+		  && (((GdkWindowObject *) real_focus_window)->event_mask
+		      & GDK_FOCUS_CHANGE_MASK))
+		generate_focus_event (real_focus_window, FALSE);
+
+	      if (((GdkWindowObject *) grab_focus_window)->event_mask 
+		  & GDK_FOCUS_CHANGE_MASK)
+		generate_focus_event (grab_focus_window, TRUE);
+	    }
+	}
+    }
   
   return return_val;
 }
@@ -743,10 +784,39 @@ void
 gdk_display_keyboard_ungrab (GdkDisplay *display,
                              guint32 time)
 {
+  GdkWindow *real_focus_window, *grab_focus_window;
+
   g_return_if_fail (display == gdk_display_get_default ());
 
   GDK_NOTE (EVENTS, g_print ("gdk_keyboard_ungrab\n"));
 
+  if (k_grab_window && !k_grab_owner_events)
+    {
+      real_focus_window = gdk_win32_handle_table_lookup ((GdkNativeWindow) GetFocus ());
+      if (real_focus_window)
+	real_focus_window = gdk_window_get_toplevel (real_focus_window);
+      if (!GDK_WINDOW_DESTROYED (k_grab_window))
+	grab_focus_window = gdk_window_get_toplevel (k_grab_window);
+      else
+	grab_focus_window = NULL;
+      if (real_focus_window != grab_focus_window)
+	{
+	  /* Generate events for focus change from grabber to the window that
+	   * really has focus. Important for example if a new window is created
+	   * while focus is grabbed.
+	   */
+	  if (grab_focus_window
+	      && (((GdkWindowObject *) grab_focus_window)->event_mask
+		  & GDK_FOCUS_CHANGE_MASK))
+	    generate_focus_event (grab_focus_window, FALSE);
+
+	  if (real_focus_window && !GDK_WINDOW_DESTROYED (real_focus_window)
+	      && (((GdkWindowObject *) real_focus_window)->event_mask
+		  & GDK_FOCUS_CHANGE_MASK))
+	    generate_focus_event (real_focus_window, TRUE);
+	}
+    }
+
   k_grab_window = NULL;
 }
 
@@ -851,7 +921,7 @@ build_key_event_state (GdkEvent *event,
    * does indicate correctly whether it is the right Control or Alt
    * key. But that would be a bit messy.
    */
-  if (!IS_WIN_NT () &&
+  if (!G_WIN32_IS_NT_BASED () &&
       _gdk_keyboard_has_altgr &&
       key_state[VK_CONTROL] & 0x80 &&
       key_state[VK_MENU] & 0x80)
@@ -2054,10 +2124,18 @@ resize_timer_proc (HWND     hwnd,
 		   UINT     id,
 		   DWORD    time)
 {
-  if (resizing)
+  if (_sizemove_in_progress)
     handle_stuff_while_moving_or_resizing ();
 }
 
+static void
+handle_display_change (void)
+{
+  _gdk_monitor_init ();
+  _gdk_root_window_size_init ();
+  g_signal_emit_by_name (_gdk_screen, "size_changed");
+}
+
 static gboolean
 gdk_event_translate (GdkDisplay *display,
 		     MSG        *msg,
@@ -2824,7 +2902,7 @@ gdk_event_translate (GdkDisplay *display,
 
     case WM_SETFOCUS:
     case WM_KILLFOCUS:
-      if (p_grab_window != NULL && !p_grab_owner_events && !(p_grab_mask & GDK_FOCUS_CHANGE_MASK))
+      if (k_grab_window != NULL && !k_grab_owner_events)
 	break;
 
       if (!(((GdkWindowObject *) window)->event_mask & GDK_FOCUS_CHANGE_MASK))
@@ -2833,12 +2911,7 @@ gdk_event_translate (GdkDisplay *display,
       if (GDK_WINDOW_DESTROYED (window))
 	break;
 
-      event = gdk_event_new (GDK_FOCUS_CHANGE);
-      event->focus_change.window = window;
-      event->focus_change.in = (msg->message == WM_SETFOCUS);
-
-      append_event (display, event);
-
+      generate_focus_event (window, (msg->message == WM_SETFOCUS));
       return_val = TRUE;
       break;
 
@@ -2966,12 +3039,12 @@ gdk_event_translate (GdkDisplay *display,
       break;
 
     case WM_ENTERSIZEMOVE:
-      resizing = TRUE;
+      _sizemove_in_progress = TRUE;
       resize_timer = SetTimer (NULL, 0, 20, resize_timer_proc);
       break;
 
     case WM_EXITSIZEMOVE:
-      resizing = FALSE;
+      _sizemove_in_progress = FALSE;
       KillTimer (NULL, resize_timer);
       break;
 
@@ -2979,7 +3052,7 @@ gdk_event_translate (GdkDisplay *display,
       /* Once we've entered the moving or sizing modal loop, we won't
        * return to the main loop until we're done sizing or moving.
        */
-      if (resizing &&
+      if (_sizemove_in_progress &&
 	 GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD &&
 	 !GDK_WINDOW_DESTROYED (window))
 	{
@@ -3276,6 +3349,11 @@ gdk_event_translate (GdkDisplay *display,
       return_val = TRUE;
       break;
 
+    case WM_DISPLAYCHANGE:
+      handle_display_change ();
+      break;
+      
+
 #ifdef HAVE_WINTAB
       /* Handle WINTAB events here, as we know that gdkinput.c will
        * use the fixed WT_DEFBASE as lcMsgBase, and we thus can use the
