@@ -665,9 +665,10 @@ _gtk_action_sync_menu_visible (GtkAction *action,
   visible = gtk_action_is_visible (action);
   hide_if_empty = action->private_data->hide_if_empty;
 
-  g_object_set (G_OBJECT (proxy),
-		"visible", visible && !(empty && hide_if_empty),
-		NULL);
+  if (visible && !(empty && hide_if_empty))
+    gtk_widget_show (proxy);
+  else
+    gtk_widget_hide (proxy);
 }
 
 gboolean _gtk_menu_is_empty (GtkWidget *menu);
@@ -869,9 +870,9 @@ connect_proxy (GtkAction     *action,
 	}
 
       if (gtk_menu_item_get_submenu (GTK_MENU_ITEM (proxy)) == NULL)
-      g_signal_connect_object (proxy, "activate",
-			       G_CALLBACK (gtk_action_activate), action,
-			       G_CONNECT_SWAPPED);
+	g_signal_connect_object (proxy, "activate",
+				 G_CALLBACK (gtk_action_activate), action,
+				 G_CONNECT_SWAPPED);
 
     }
   else if (GTK_IS_TOOL_ITEM (proxy))
@@ -903,6 +904,8 @@ connect_proxy (GtkAction     *action,
 			       G_CALLBACK (gtk_action_create_menu_proxy),
 			       action, 0);
 
+      gtk_tool_item_rebuild_menu (GTK_TOOL_ITEM (proxy));
+
       /* toolbar button specific synchronisers ... */
       if (GTK_IS_TOOL_BUTTON (proxy))
 	{
@@ -920,10 +923,10 @@ connect_proxy (GtkAction     *action,
 	  g_signal_connect_object (action, "notify::stock-id",
 				   G_CALLBACK (gtk_action_sync_property), 
 				   proxy, 0);
-      g_signal_connect_object (proxy, "clicked",
-			       G_CALLBACK (gtk_action_activate), action,
-			       G_CONNECT_SWAPPED);
-    }
+	  g_signal_connect_object (proxy, "clicked",
+				   G_CALLBACK (gtk_action_activate), action,
+				   G_CONNECT_SWAPPED);
+	}
     }
   else if (GTK_IS_BUTTON (proxy))
     {
@@ -1190,7 +1193,7 @@ gtk_action_get_proxies (GtkAction *action)
  *
  * Since: 2.4
  **/
-const gchar *
+G_CONST_RETURN gchar *
 gtk_action_get_name (GtkAction *action)
 {
   g_return_val_if_fail (GTK_IS_ACTION (action), NULL);
@@ -1435,6 +1438,30 @@ gtk_action_set_accel_path (GtkAction   *action,
   action->private_data->accel_quark = g_quark_from_string (accel_path);
 }
 
+/**
+ * gtk_action_get_accel_path:
+ * @action: the action object
+ *
+ * Returns the accel path for this action.  
+ *
+ * Since: 2.6
+ *
+ * Returns: the accel path for this action, or %NULL
+ *   if none is set. The returned string is owned by GTK+
+ *   and must not be freed or modified.
+ */
+G_CONST_RETURN gchar *
+gtk_action_get_accel_path (GtkAction *action)
+{
+  g_return_val_if_fail (GTK_IS_ACTION (action), NULL);
+
+  if (action->private_data->accel_quark)
+    return g_quark_to_string (action->private_data->accel_quark);
+  else
+    return NULL;
+}
+
+
 /**
  * gtk_action_set_accel_group:
  * @action: the action object
