@@ -387,7 +387,7 @@ gtk_action_group_real_get_action (GtkActionGroup *self,
  * 
  * Since: 2.4
  */
-const gchar *
+G_CONST_RETURN gchar *
 gtk_action_group_get_name (GtkActionGroup *action_group)
 {
   g_return_val_if_fail (GTK_IS_ACTION_GROUP (action_group), NULL);
@@ -456,7 +456,7 @@ gtk_action_group_set_sensitive (GtkActionGroup *action_group, gboolean sensitive
  * they are visible (see gtk_action_get_visible()) and their group
  * is visible.
  * 
- * Return value: %TRUE if the group is sensitive.
+ * Return value: %TRUE if the group is visible.
  * 
  * Since: 2.4
  */
