@@ -198,8 +198,8 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    * @widget: the object which received the signal
    * @prefix: the common prefix of all possible completions
    * 
-   * The ::insert-prefix signal is emitted when the inline autocompletion
-   * is triggered. The default behaviour is to make the entry display the 
+   * Gets emitted when the inline autocompletion is triggered. 
+   * The default behaviour is to make the entry display the 
    * whole prefix and select the newly inserted part.
    *
    * Applications may connect to this signal in order to insert only a
@@ -227,10 +227,10 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    * @model: the #GtkTreeModel containing the matches
    * @iter: a #GtkTreeIter positioned at the selected match
    * 
-   * The ::match-selected signal is emitted when a match from the list
-   * is selected. The default behaviour is to replace the contents of the
-   * entry with the contents of the text column in the row pointed to by
-   * @iter.
+   * Gets emitted when a match from the list is selected. 
+   * The default behaviour is to replace the contents of the 
+   * entry with the contents of the text column in the row 
+   * pointed to by @iter.
    *
    * Return value: %TRUE if the signal has been handled
    * 
@@ -252,8 +252,7 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    * @widget: the object which received the signal
    * @index: the index of the activated action
    *
-   * The ::action-activated signal is emitted when an action
-   * is activated.
+   * Gets emitted when an action is activated.
    * 
    * Since: 2.4
    */
@@ -301,11 +300,10 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
                                                      G_PARAM_READWRITE));
 
   /**
-   * GtkEntryCompletion:inline_completion:
+   * GtkEntryCompletion:inline-completion:
    * 
-   * The boolean :inline_completion property determines whether the
-   * common prefix of the possible completions should be inserted 
-   * automatically in the entry.
+   * Determines whether the common prefix of the possible completions 
+   * should be inserted automatically in the entry.
    *
    * Since: 2.6
    **/
@@ -317,10 +315,10 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
  							 FALSE,
  							 G_PARAM_READWRITE));
   /**
-   * GtkEntryCompletion:popup_completion:
+   * GtkEntryCompletion:popup-completion:
    * 
-   * The boolean :popup_completion property determines whether the
-   * possible completions should be shown in a popup window. 
+   * Determines whether the possible completions should be 
+   * shown in a popup window. 
    *
    * Since: 2.6
    **/
@@ -935,6 +933,15 @@ gtk_entry_completion_set_model (GtkEntryCompletion *completion,
   g_return_if_fail (GTK_IS_ENTRY_COMPLETION (completion));
   g_return_if_fail (model == NULL || GTK_IS_TREE_MODEL (model));
 
+  if (!model)
+    {
+      gtk_tree_view_set_model (GTK_TREE_VIEW (completion->priv->tree_view),
+			       NULL);
+      _gtk_entry_completion_popdown (completion);
+      completion->priv->filter_model = NULL;
+      return;
+    }
+     
   /* code will unref the old filter model (if any) */
   completion->priv->filter_model =
     GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (model, NULL));
@@ -967,6 +974,9 @@ gtk_entry_completion_get_model (GtkEntryCompletion *completion)
 {
   g_return_val_if_fail (GTK_IS_ENTRY_COMPLETION (completion), NULL);
 
+  if (!completion->priv->filter_model)
+    return NULL;
+  
   return gtk_tree_model_filter_get_model (completion->priv->filter_model);
 }
 
@@ -1055,8 +1065,10 @@ gtk_entry_completion_complete (GtkEntryCompletion *completion)
   gchar *tmp;
 
   g_return_if_fail (GTK_IS_ENTRY_COMPLETION (completion));
-  g_return_if_fail (completion->priv->filter_model != NULL);
 
+  if (!completion->priv->filter_model)
+    return;
+  
   if (completion->priv->case_normalized_key)
     g_free (completion->priv->case_normalized_key);
 
@@ -1221,39 +1233,6 @@ gtk_entry_completion_get_text_column (GtkEntryCompletion *completion)
 
 /* private */
 
-/* lame copy from gtkentry.c */
-static void
-get_borders (GtkEntry *entry,
-             gint     *xborder,
-             gint     *yborder)
-{
-  GtkWidget *widget = GTK_WIDGET (entry);
-  gint focus_width;
-  gboolean interior_focus;
-
-  gtk_widget_style_get (widget,
-                        "interior-focus", &interior_focus,
-                        "focus-line-width", &focus_width,
-                        NULL);
-
-  if (entry->has_frame)
-    {
-      *xborder = widget->style->xthickness;
-      *yborder = widget->style->ythickness;
-    }
-  else
-    {
-      *xborder = 0;
-      *yborder = 0;
-    }
-
-  if (!interior_focus)
-    {
-      *xborder += focus_width;
-      *yborder += focus_width;
-    }
-}
-
 static gboolean
 gtk_entry_completion_list_enter_notify (GtkWidget        *widget,
 					GdkEventCrossing *event,
@@ -1291,9 +1270,12 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
   GtkTreePath *path;
   gboolean above;
   gint width;
-  
+
+  if (!completion->priv->entry->window)
+    return FALSE;
+
   gdk_window_get_origin (completion->priv->entry->window, &x, &y);
-  get_borders (GTK_ENTRY (completion->priv->entry), &x_border, &y_border);
+  _gtk_entry_get_borders (GTK_ENTRY (completion->priv->entry), &x_border, &y_border);
 
   matches = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (completion->priv->filter_model), NULL);
 
@@ -1312,9 +1294,8 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
 						  GTK_WIDGET (completion->priv->entry)->window);
   gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
-  width = MIN (completion->priv->entry->allocation.width, monitor.width);
-  gtk_widget_set_size_request (completion->priv->tree_view,
-                               width - 2 * x_border, items * height);
+  width = MIN (completion->priv->entry->allocation.width, monitor.width) - 2 * x_border;
+  gtk_widget_set_size_request (completion->priv->tree_view, width, items * height);
 
   /* default on no match */
   completion->priv->current_selected = -1;
@@ -1324,13 +1305,7 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
   if (items)
     {
       gtk_widget_show (completion->priv->action_view);
-
-      gtk_tree_view_column_cell_get_size (gtk_tree_view_get_column (GTK_TREE_VIEW (completion->priv->action_view), 0),
-                                          NULL, NULL, NULL, NULL,
-                                          &height);
-
-      gtk_widget_set_size_request (completion->priv->action_view,
-                                   width - 2 * x_border, items * height);
+      gtk_widget_set_size_request (completion->priv->action_view, width, -1);
     }
   else
     gtk_widget_hide (completion->priv->action_view);
@@ -1376,6 +1351,9 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
   if (GTK_WIDGET_MAPPED (completion->priv->popup_window))
     return;
 
+  if (!GTK_WIDGET_MAPPED (completion->priv->entry))
+    return;
+
   completion->priv->ignore_enter = TRUE;
     
   column = gtk_tree_view_get_column (GTK_TREE_VIEW (completion->priv->action_view), 0);
@@ -1391,7 +1369,7 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
   _gtk_entry_completion_resize_popup (completion);
 
   gtk_widget_show (completion->priv->popup_window);
-
+    
   gtk_grab_add (completion->priv->popup_window);
   gdk_pointer_grab (completion->priv->popup_window->window, TRUE,
                     GDK_BUTTON_PRESS_MASK |
