@@ -38,11 +38,11 @@ extern "C" {
 
 
 #define GTK_TYPE_LABEL		  (gtk_label_get_type ())
-#define GTK_LABEL(obj)		  (GTK_CHECK_CAST ((obj), GTK_TYPE_LABEL, GtkLabel))
-#define GTK_LABEL_CLASS(klass)	  (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_LABEL, GtkLabelClass))
-#define GTK_IS_LABEL(obj)	  (GTK_CHECK_TYPE ((obj), GTK_TYPE_LABEL))
-#define GTK_IS_LABEL_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_LABEL))
-#define GTK_LABEL_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_LABEL, GtkLabelClass))
+#define GTK_LABEL(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_LABEL, GtkLabel))
+#define GTK_LABEL_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_LABEL, GtkLabelClass))
+#define GTK_IS_LABEL(obj)	  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_LABEL))
+#define GTK_IS_LABEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_LABEL))
+#define GTK_LABEL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_LABEL, GtkLabelClass))
        
 
 typedef struct _GtkLabel       GtkLabel;
@@ -60,6 +60,7 @@ struct _GtkLabel
   guint   wrap : 1;
   guint   use_underline : 1;
   guint   use_markup : 1;
+  guint   ellipsize : 3;
 
   guint   mnemonic_keyval;
   
@@ -96,7 +97,7 @@ struct _GtkLabelClass
   void (*_gtk_reserved4) (void);
 };
 
-GtkType               gtk_label_get_type          (void) G_GNUC_CONST;
+GType                 gtk_label_get_type          (void) G_GNUC_CONST;
 GtkWidget*            gtk_label_new               (const char    *str);
 GtkWidget*            gtk_label_new_with_mnemonic (const char    *str);
 void                  gtk_label_set_text          (GtkLabel      *label,
@@ -128,6 +129,12 @@ void     gtk_label_set_text_with_mnemonic         (GtkLabel         *label,
 void     gtk_label_set_justify                    (GtkLabel         *label,
 						   GtkJustification  jtype);
 GtkJustification gtk_label_get_justify            (GtkLabel         *label);
+void     gtk_label_set_ellipsize		  (GtkLabel         *label,
+						   PangoEllipsizeMode mode);
+PangoEllipsizeMode gtk_label_get_ellipsize        (GtkLabel         *label);
+void     gtk_label_set_width_chars		  (GtkLabel         *label,
+						   gint              n_chars);
+gint     gtk_label_get_width_chars                (GtkLabel         *label);
 void     gtk_label_set_pattern                    (GtkLabel         *label,
 						   const gchar      *pattern);
 void     gtk_label_set_line_wrap                  (GtkLabel         *label,
@@ -148,6 +155,9 @@ void         gtk_label_get_layout_offsets (GtkLabel *label,
                                            gint     *x,
                                            gint     *y);
 
+void         gtk_label_set_single_line_mode  (GtkLabel *label,
+                                              gboolean single_line_mode);
+gboolean     gtk_label_get_single_line_mode  (GtkLabel *label);
 
 #ifndef GTK_DISABLE_DEPRECATED
 
