@@ -157,7 +157,9 @@ static const GDebugKey gtk_debug_keys[] = {
   {"updates", GTK_DEBUG_UPDATES},
   {"keybindings", GTK_DEBUG_KEYBINDINGS},
   {"multihead", GTK_DEBUG_MULTIHEAD},
-  {"modules", GTK_DEBUG_MODULES}
+  {"modules", GTK_DEBUG_MODULES},
+  {"geometry", GTK_DEBUG_GEOMETRY},
+  {"icontheme", GTK_DEBUG_ICONTHEME}
 };
 
 static const guint gtk_ndebug_keys = sizeof (gtk_debug_keys) / sizeof (GDebugKey);
@@ -328,7 +330,8 @@ static gboolean do_setlocale = TRUE;
 /**
  * gtk_disable_setlocale:
  * 
- * Prevents gtk_init() and gtk_init_check() from automatically
+ * Prevents gtk_init(), gtk_init_check(), gtk_init_with_args() and
+ * gtk_parse_args() from automatically
  * calling <literal>setlocale (LC_ALL, "")</literal>. You would 
  * want to use this function if you wanted to set the locale for 
  * your program to something other than the user's locale, or if 
@@ -350,6 +353,7 @@ gtk_disable_setlocale (void)
 #endif
 
 static GString *gtk_modules_string = NULL;
+static gboolean g_fatal_warnings = FALSE;
 
 #ifdef G_ENABLE_DEBUG
 static gboolean
@@ -389,23 +393,11 @@ gtk_arg_module_cb (const char *key, const char *value, gpointer user_data)
   return TRUE;
 }
 
-static gboolean
-gtk_arg_warnings_cb (const char *key, const char *value, gpointer user_data)
-{
-  GLogLevelFlags fatal_mask;
-
-  fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
-  fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
-  g_log_set_always_fatal (fatal_mask);
-
-  return TRUE;
-}
-
 static GOptionEntry gtk_args[] = {
   { "gtk-module",       0, 0, G_OPTION_ARG_CALLBACK, gtk_arg_module_cb,   
     /* Description of --gtk-module=MODULES in --help output */ N_("Load additional GTK+ modules"), 
     /* Placeholder in --gtk-module=MODULES in --help output */ N_("MODULES") },
-  { "g-fatal-warnings", 0, 0, G_OPTION_ARG_CALLBACK, gtk_arg_warnings_cb, 
+  { "g-fatal-warnings", 0, 0, G_OPTION_ARG_NONE, &g_fatal_warnings, 
     /* Description of --g-fatal-warnings in --help output */   N_("Make all warnings fatal"), NULL },
 #ifdef G_ENABLE_DEBUG
   { "gtk-debug",        0, 0, G_OPTION_ARG_CALLBACK, gtk_arg_debug_cb,    
@@ -460,6 +452,15 @@ static void
 do_post_parse_initialization (int    *argc,
 			      char ***argv)
 {
+  if (g_fatal_warnings)
+    {
+      GLogLevelFlags fatal_mask;
+
+      fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
+      fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
+      g_log_set_always_fatal (fatal_mask);
+    }
+
   if (gtk_debug_flags & GTK_DEBUG_UPDATES)
     gdk_window_set_debug_updates (TRUE);
 
@@ -537,6 +538,21 @@ post_parse_hook (GOptionContext *context,
 }
 
 
+/**
+ * gtk_get_option_group:
+ * @open_default_display: whether to open the default display 
+ *    when parsing the commandline arguments
+ * 
+ * Returns a #GOptionGroup for the commandline arguments recognized
+ * by GTK+ and GDK. You should add this group to your #GOptionContext 
+ * with g_option_context_add_group(), if you are using 
+ * g_option_context_parse() to parse your commandline arguments.
+ *
+ * Returns a #GOptionGroup for the commandline arguments recognized
+ *   by GTK+
+ *
+ * Since: 2.6
+ */
 GOptionGroup *
 gtk_get_option_group (gboolean open_default_display)
 {
@@ -556,6 +572,31 @@ gtk_get_option_group (gboolean open_default_display)
   return group;
 }
 
+/**
+ * gtk_init_with_args:
+ * @argc: a pointer to the number of command line arguments.
+ * @argv: a pointer to the array of command line arguments.
+ * @parameter_string: a string which is displayed in
+ *    the first line of <option>--help</option> output, after 
+ *    <literal><replaceable>programname</replaceable> [OPTION...]</literal>
+ * @entries: a %NULL-terminated array of #GOptionEntry<!-- -->s
+ *    describing the options of your program
+ * @translation_domain: a translation domain to use for translating
+ *    the <option>--help</option> output for the options in @entries
+ *    with gettext(), or %NULL
+ * @error: a return location for errors 
+ *
+ * This function does the same work as gtk_init_check(). 
+ * Additionally, it allows you to add your own commandline options, 
+ * and it automatically generated nicely formatted 
+ * <option>--help</option> output. Note that your program will
+ * be terminated after writing out the help output.
+ *
+ * Returns: %TRUE if the GUI has been successfully initialized, 
+ *               %FALSE otherwise.
+ * 
+ * Since: 2.6
+ */
 gboolean
 gtk_init_with_args (int            *argc,
 		    char         ***argv,
@@ -567,10 +608,13 @@ gtk_init_with_args (int            *argc,
   GOptionContext *context;
   GOptionGroup *gtk_group;
   gboolean retval;
-  
+
   if (gtk_initialized)
     return TRUE;
 
+  if (!check_setugid ())
+    return FALSE;
+
   gtk_group = gtk_get_option_group (TRUE);
   
   context = g_option_context_new (parameter_string);
@@ -578,9 +622,8 @@ gtk_init_with_args (int            *argc,
   
   if (entries)
     g_option_context_add_main_entries (context, entries, translation_domain);
-
   retval = g_option_context_parse (context, argc, argv, error);
-
+  
   g_option_context_free (context);
 
   return retval;
@@ -609,6 +652,7 @@ gtk_parse_args (int    *argc,
 		char ***argv)
 {
   GOptionContext *option_context;
+  GOptionGroup *gtk_group;
   
   if (gtk_initialized)
     return TRUE;
@@ -616,19 +660,14 @@ gtk_parse_args (int    *argc,
   if (!check_setugid ())
     return FALSE;
 
-  do_pre_parse_initialization (argc, argv);
-
   option_context = g_option_context_new (NULL);
   g_option_context_set_ignore_unknown_options (option_context, TRUE);
   g_option_context_set_help_enabled (option_context, FALSE);
-  
-  g_option_context_add_main_entries (option_context, gtk_args, GETTEXT_PACKAGE);
-
+  gtk_group = gtk_get_option_group (FALSE);
+  g_option_context_set_main_group (option_context, gtk_group);
   g_option_context_parse (option_context, argc, argv, NULL);
   g_option_context_free (option_context);
 
-  do_post_parse_initialization (argc, argv);
-  
   return TRUE;
 }
 
@@ -678,7 +717,13 @@ gtk_init_check (int	 *argc,
  * applications.  It will initialize everything needed to operate the 
  * toolkit and parses some standard command line options. @argc and 
  * @argv are adjusted accordingly so your own code will 
- * never see those standard arguments.
+ * never see those standard arguments. 
+ *
+ * Note that there are some alternative ways to initialize GTK+: 
+ * if you are calling gtk_parse_args(), gtk_init_check(), 
+ * gtk_init_with_args() or g_option_context_parse() with 
+ * the option group returned by gtk_get_option_group(), you 
+ * <emphasis>don't</emphasis> have to call gtk_init().
  *
  * <note><para>
  * This function will terminate your program if it was unable to initialize 
@@ -956,12 +1001,16 @@ gtk_main (void)
 
       gdk_flush ();
     }
-	      
+    
   main_loops = g_slist_remove (main_loops, loop);
 
   g_main_loop_unref (loop);
 
   gtk_main_loop_level--;
+
+  /* Try storing all clipboard data we have */
+  if (gtk_main_loop_level == 0)
+    _gtk_clipboard_store_all ();
 }
 
 guint
