@@ -24,9 +24,13 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#undef GTK_DISABLE_DEPRECATED
+
+#include <config.h>
 #include <string.h>
 #include "gdk/gdkkeysyms.h"
 #include "gdk/gdki18n.h"
+#include "gtkalias.h"
 #include "gtkclipboard.h"
 #include "gtkoldeditable.h"
 #include "gtkmain.h"
@@ -142,9 +146,9 @@ gtk_old_editable_get_type (void)
 	NULL,		/* class_finalize */
 	NULL,		/* class_data */
 	sizeof (GtkOldEditable),
-	0,              /* n_preallocs */
+	0,		/* n_preallocs */
         (GInstanceInitFunc) gtk_old_editable_init,
-	NULL            /* value_table */
+	NULL,		/* value_table */
       };
 
       static const GInterfaceInfo editable_info =
@@ -715,7 +719,7 @@ old_editable_text_received_cb (GtkClipboard *clipboard,
  * @old_editable: a #GtkOldEditable
  * @claim: if %TRUE, claim ownership of the selection, if %FALSE, give
  *   up ownership
- * @time: timestamp for this operation
+ * @time_: timestamp for this operation
  * 
  * Claims or gives up ownership of the selection.
  */
