@@ -95,7 +95,13 @@ void           gtk_target_list_add       (GtkTargetList  *list,
 				  	  GdkAtom         target,
 					  guint           flags,
 					  guint           info);
-void           gtk_target_list_add_text_targets (GtkTargetList  *list);
+void           gtk_target_list_add_text_targets  (GtkTargetList  *list,
+						  guint           info);
+void           gtk_target_list_add_image_targets (GtkTargetList  *list,
+						  guint           info,
+						  gboolean        writable);
+void           gtk_target_list_add_uri_targets   (GtkTargetList  *list,
+						  guint           info);
 void           gtk_target_list_add_table (GtkTargetList        *list,
 					  const GtkTargetEntry *targets,
 					  guint                 ntargets);
@@ -138,11 +144,19 @@ gboolean gtk_selection_data_set_text (GtkSelectionData     *selection_data,
 				      const gchar          *str,
 				      gint                  len);
 guchar * gtk_selection_data_get_text (GtkSelectionData     *selection_data);
+gboolean gtk_selection_data_set_pixbuf   (GtkSelectionData  *selection_data,
+				          GdkPixbuf         *pixbuf);
+GdkPixbuf *gtk_selection_data_get_pixbuf (GtkSelectionData  *selection_data);
+gboolean gtk_selection_data_set_uris (GtkSelectionData     *selection_data,
+				      gchar               **uris);
+gchar  **gtk_selection_data_get_uris (GtkSelectionData     *selection_data);
 
 gboolean gtk_selection_data_get_targets          (GtkSelectionData  *selection_data,
 						  GdkAtom          **targets,
 						  gint              *n_atoms);
 gboolean gtk_selection_data_targets_include_text (GtkSelectionData  *selection_data);
+gboolean gtk_selection_data_targets_include_image (GtkSelectionData  *selection_data,
+						   gboolean           writable);
 
 /* Called when a widget is destroyed */
 
