@@ -24,6 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
+#include "gtkalias.h"
 #include "gtktextutil.h"
 #include "gtkintl.h"
 #include "gtkmenuitem.h"
