@@ -37,8 +37,10 @@
  * we warned you and we're not liable for any head injuries.
  */
 
+#include <config.h>
 #include <string.h>
 
+#include "gtkalias.h"
 #include "gtktreemodelsort.h"
 #include "gtktreesortable.h"
 #include "gtktreestore.h"
