@@ -20,13 +20,14 @@
 #include <config.h>
 #include <string.h>
 #include <gobject/gvaluecollector.h>
+#include "gtkalias.h"
 #include "gtktreemodel.h"
 #include "gtktreestore.h"
 #include "gtktreedatalist.h"
 #include "gtktreednd.h"
 
 #define G_NODE(node) ((GNode *)node)
-#define GTK_TREE_STORE_IS_SORTED(tree) (GTK_TREE_STORE (tree)->sort_column_id != -2)
+#define GTK_TREE_STORE_IS_SORTED(tree) (GTK_TREE_STORE (tree)->sort_column_id != GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID)
 #define VALID_ITER(iter, tree_store) (iter!= NULL && iter->user_data != NULL && tree_store->stamp == iter->stamp)
 
 static void         gtk_tree_store_init            (GtkTreeStore      *tree_store);
@@ -965,7 +966,7 @@ gtk_tree_store_set_valist (GtkTreeStore *tree_store,
 	}
     }
 
-  if (func != gtk_tree_data_list_compare_func)
+  if (func != _gtk_tree_data_list_compare_func)
     maybe_need_sort = TRUE;
 
   while (column != -1)
@@ -998,7 +999,7 @@ gtk_tree_store_set_valist (GtkTreeStore *tree_store,
 						   &value,
 						   FALSE) || emit_signal;
 
-      if (func == gtk_tree_data_list_compare_func &&
+      if (func == _gtk_tree_data_list_compare_func &&
 	  column == tree_store->sort_column_id)
 	maybe_need_sort = TRUE;
 
@@ -2171,7 +2172,7 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
 
       if (gtk_tree_path_get_depth (path) != gtk_tree_path_get_depth (pos_path))
         {
-          g_warning ("Given childs are not in the same level\n");
+          g_warning ("Given children are not in the same level\n");
 
 	  goto free_paths_and_out;
         }
@@ -2183,7 +2184,7 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
       if (gtk_tree_path_get_depth (path) > 0 &&
 	  gtk_tree_path_compare (path, tmppath))
         {
-          g_warning ("Given childs are not in the same level\n");
+          g_warning ("Given children are not in the same level\n");
 
           gtk_tree_path_free (tmppath);
 	  goto free_paths_and_out;
@@ -2554,8 +2555,6 @@ gtk_tree_store_sort_helper (GtkTreeStore *tree_store,
       return;
     }
 
-  g_assert (GTK_TREE_STORE_IS_SORTED (tree_store));
-
   list_length = 0;
   for (tmp_node = node; tmp_node; tmp_node = tmp_node->next)
     list_length++;
@@ -2614,6 +2613,9 @@ gtk_tree_store_sort_helper (GtkTreeStore *tree_store,
 static void
 gtk_tree_store_sort (GtkTreeStore *tree_store)
 {
+  if (!GTK_TREE_STORE_IS_SORTED (tree_store))
+    return;
+
   if (tree_store->sort_column_id != -1)
     {
       GtkTreeDataSortHeader *header = NULL;
@@ -2673,7 +2675,7 @@ gtk_tree_store_sort_iter_changed (GtkTreeStore *tree_store,
     }
 
   /* If it's the built in function, we don't sort. */
-  if (func == gtk_tree_data_list_compare_func &&
+  if (func == _gtk_tree_data_list_compare_func &&
       tree_store->sort_column_id != column)
     return;
 
@@ -2856,19 +2858,22 @@ gtk_tree_store_set_sort_column_id (GtkTreeSortable  *sortable,
       (tree_store->order == order))
     return;
 
-  if (sort_column_id != GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
+  if (sort_column_id != GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID)
     {
-      GtkTreeDataSortHeader *header = NULL;
+      if (sort_column_id != GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
+	{
+	  GtkTreeDataSortHeader *header = NULL;
 
-      header = _gtk_tree_data_list_get_header (tree_store->sort_list, sort_column_id);
+	  header = _gtk_tree_data_list_get_header (tree_store->sort_list, sort_column_id);
 
-      /* We want to make sure that we have a function */
-      g_return_if_fail (header != NULL);
-      g_return_if_fail (header->func != NULL);
-    }
-  else
-    {
-      g_return_if_fail (tree_store->default_sort_func != NULL);
+	  /* We want to make sure that we have a function */
+	  g_return_if_fail (header != NULL);
+	  g_return_if_fail (header->func != NULL);
+	}
+      else
+	{
+	  g_return_if_fail (tree_store->default_sort_func != NULL);
+	}
     }
 
   tree_store->sort_column_id = sort_column_id;
